import { PackageManagerCommands } from '../../utils/package-manager';
export declare function runPackageManagerInstallPlugins(repoRoot: string, pmc: PackageManagerCommands, plugins: string[]): void;
/**
 * Installs a plugin by running its init generator. It will change the file system tree passed in.
 * @param plugin The name of the plugin to install
 * @param repoRoot repo root
 * @param verbose verbose
 * @param options options passed to init generator
 * @returns void
 */
export declare function installPlugin(plugin: string, repoRoot: string, verbose: boolean, options: {
    [k: string]: any;
}): Promise<void>;
/**
 * Install plugins
 * Get the implementation of the plugin's init generator and run it
 * @returns a list of succeeded plugins and a map of failed plugins to errors
 */
export declare function installPlugins(plugins: string[], updatePackageScripts: boolean, repoRoot?: string, verbose?: boolean): Promise<{
    succeededPlugins: string[];
    failedPlugins: {
        [plugin: string]: Error;
    };
}>;
/**
 * Configures plugins, installs them, and outputs the results
 * @returns a list of succeeded plugins and a map of failed plugins to errors
 */
export declare function configurePlugins(plugins: string[], updatePackageScripts: boolean, pmc: PackageManagerCommands, repoRoot?: string, verbose?: boolean): Promise<{
    succeededPlugins: string[];
    failedPlugins: {
        [plugin: string]: Error;
    };
}>;
export declare function getFailedToInstallPluginErrorMessages(e: any): string[];
