"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackageEntryPointsToProjectMap = getPackageEntryPointsToProjectMap;
const posix_1 = require("node:path/posix");
function getPackageEntryPointsToProjectMap(projects) {
    const result = {};
    for (const project of Object.values(projects)) {
        const metadata = 'data' in project ? project.data.metadata : project.metadata;
        if (!metadata?.js) {
            continue;
        }
        const { packageName, packageExports } = metadata.js;
        if (!packageExports || typeof packageExports === 'string') {
            // no `exports` or it points to a file, which would be the equivalent of
            // an '.' export, in which case the package name is the entry point
            result[packageName] = project;
        }
        else {
            for (const entryPoint of Object.keys(packageExports)) {
                result[(0, posix_1.join)(packageName, entryPoint)] = project;
            }
        }
    }
    return result;
}
